@@ -52,7 +52,7 @@ plotGMPhyloMorphoSpace<-function(phy,A,tip.labels=TRUE,node.labels=TRUE,ancState
     if(is.null(dimnames(A)[[1]])){
       stop("Data matrix does not include taxa names as dimnames for rows.")  }
     x<-A }
-  if (class(phy) != "phylo") 
+  if (!inherits(phy, "phylo"))
     stop("tree must be of class 'phylo.'")
   if (!is.binary.tree(phy)) 
     stop("tree is not fully bifurcating. Consider 'multi2di' in ape.")
