@@ -84,10 +84,11 @@
 #'   \item{means}{The observed least squares means based on the linear model.}
 #'   \item{pc.means}{The observed least squares means rotated to their principal components.}
 #'   \item{pc.data}{The observed data rotated to the principal components calculated from the 
-#'   covariance matrix among means.  In the case that trajectories are input as data, pc.data is a matrix of
+#'   covariance matrix among means. In the case that trajectories are input as data, pc.data is a matrix of
 #'    the trajectories rotated to align with principal axes in the data space.}
+#'   \item{pc.summary}{A table summarizing the percent variation explained by each pc axis, equivalent to summary of \code{\link{prcomp}}.}
 #'   \item{pc.trajectories}{The observed trajectories rotated to the principal components calculated from the 
-#'   covariance matrix among means.  In the case that trajectories are input as data, pc.trajectories is a list of the
+#'   covariance matrix among means. In the case that trajectories are input as data, pc.trajectories is a list of the
 #'   the rows of pc.data as separate matrices.}
 #'   \item{random.means}{A list of matrices of means calculated in the random permutations.}
 #'   \item{random.trajectories}{A list of all random means reconfigured as trajectories. The observed
@@ -171,6 +172,7 @@
 #' pt.seq.pattern = c("green", "gray30", "red"), pt.scale = 1.3)
 trajectory.analysis <- function(f1, f2=NULL, iter=999, seed=NULL, traj.pts = NULL, 
                                 data = NULL,print.progress=TRUE){
+  if(!is.null(data)) data <- droplevels(data)
   pfit1 <- procD.fit(f1, data=data, pca=FALSE)
   Terms <- pfit1$Terms
   dat <- pfit1$data
@@ -244,6 +246,7 @@ trajectory.analysis <- function(f1, f2=NULL, iter=999, seed=NULL, traj.pts = NUL
     colnames(Z.MD) <- colnames(Z.angle) <- colnames(Z.SD) <- gp.names
   out <- list(aov.table = pda$aov.table, 
               means = means, pc.means =pc.means, pc.data = pc.data,
+              pc.summary = summary(pca.means),
               pc.trajectories = pc.trajectories,
               random.means = pta$means,
               random.trajectories = pta$trajectories,
@@ -254,7 +257,7 @@ trajectory.analysis <- function(f1, f2=NULL, iter=999, seed=NULL, traj.pts = NUL
               trajectory.angle.deg = Tang*180/pi,
               trajectory.shape.dist = SD,
               P.magnitude.diff = P.MD,
-              P.angle = P.angle,
+              P.angle.diff = P.angle,
               P.shape.diff = P.SD,
               Z.magnitude.diff = Z.MD,
               Z.angle = Z.angle,
